#include <algorithm>
#include <cstdlib>
#include <fstream>
#include <iostream>

#include <rlcsa/misc/utils.h>

#include "pdltopk.h"


int
main(int argc, char** argv)
{
  std::cout << "Building PDL-topk" << std::endl;
  if(argc < 3)
  {
    std::cout << "Usage: build_topk mode base_name" << std::endl;
    std::cout << "  -m  Support merging" << std::endl;
    std::cout << "  -t  Do not support merging (default)" << std::endl;
    return 1;
  }

  bool merging = false;
  if(argv[1][0] == '-')
  {
    switch(argv[1][1])
    {
      case 'm':
        std::cout << "Mode: merge" << std::endl;
        merging = true; break;
      case 't':
        std::cout << "Mode: topk" << std::endl;
        merging = false; break;
      default:
        std::cerr << "build_topk: Invalid mode: " << argv[1] << "!" << std::endl;
        break;
    }
  }

  std::string base_name = argv[2];
  std::cout << "Base name: " << base_name << std::endl;
  std::cout << std::endl;

  RLCSA rlcsa(base_name);
  if(!(rlcsa.isOk())) { return 2; }
  rlcsa.printInfo();
  usint rlcsa_bytes = rlcsa.reportSize(true);
  usint size = rlcsa.getSize();

  PDLTopk topk(rlcsa, base_name, true, merging);
  if(!(topk.isOk())) { return 3; }
  usint bytes = topk.reportSize(false);
  usint freq_bytes = topk.reportSize(true);
  topk.writeTo(base_name);

  printSize("PDL", size, rlcsa_bytes, bytes);
  printSize("PDL+freqs", size, rlcsa_bytes, freq_bytes);
  std::cout << std::endl;

  return 0;
}
